using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace WgSoup {
	public class SliderConstChange : MVRScript {

		// IMPORTANT - DO NOT make custom enums. The dynamic C# complier crashes Unity when it encounters these for
		// some reason
        protected JSONStorable sliderValue;
        protected JSONStorableFloat sliderMinJSON;
        protected JSONStorableFloat sliderMaxJSON;
        protected JSONStorableFloat sliderPushJSON;
        protected JSONStorableFloat sliderChangeJSON;
        protected JSONStorableFloat sliderQuickJSON;
        protected JSONStorableFloat currentValueJSON;
        protected JSONStorableFloat delayValueJSON;
        protected float timer=0;
        protected float pushInitialVal=0;
        protected float pushFinalVal=0;
        protected float pushDelay=0;
        

        
		// IMPORTANT - DO NOT OVERRIDE Awake() as it is used internally by MVRScript - instead use Init() function which
		// is called right after creation
		public override void Init() {
			try {
                //UI creation
                if(containingAtom.type == "UISlider")
                {
                sliderMinJSON = new JSONStorableFloat("Slider MIN Value", 0.0f, 0f, 1f, true, true);
                sliderMaxJSON = new JSONStorableFloat("Slider MAX Value", 1f, 0f , 1f, true, true);
                sliderChangeJSON = new JSONStorableFloat("Slider CHANGE Value", -0.1f, -1f, 1f, false, true);
                sliderPushJSON = new JSONStorableFloat("Slider PUSH Value", 0.05f, 0f, 1f, true, true);
                sliderQuickJSON = new JSONStorableFloat("PUSH Quickness", 0.01f, 0.01f, 10f, true, true);
                delayValueJSON = new JSONStorableFloat("PUSH Delay Value", 0f, 0f, 10f, true, true);
                currentValueJSON = new JSONStorableFloat("Slider Value (show)", 0f, 0f, 1f, true, true);
                
                RegisterFloat(sliderMinJSON);
                CreateSlider(sliderMinJSON);
                RegisterFloat(sliderMaxJSON);
                CreateSlider(sliderMaxJSON);
                RegisterFloat(sliderChangeJSON);
                CreateSlider(sliderChangeJSON);
                RegisterFloat(sliderPushJSON);
                CreateSlider(sliderPushJSON);
                RegisterFloat(sliderQuickJSON);
                CreateSlider(sliderQuickJSON);
                RegisterFloat(delayValueJSON);
                CreateSlider(delayValueJSON);
                CreateSlider(currentValueJSON);

				var Raise_Slider = CreateButton("PUSH value", false);
                Raise_Slider.button.onClick.AddListener(Push_Slider_Foo);
                JSONStorableAction Raise_Slider_Action = new JSONStorableAction("PUSH value", () => {Push_Slider_Foo();});
                RegisterAction(Raise_Slider_Action);
                }

			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
                //Check if the plugin is assigned to a slider
                if(containingAtom.type != "UISlider")
                {
                    SuperController.LogError("Please attach to a slider trigger");
                }
                else
                {
                    sliderValue=containingAtom.GetStorableByID("Trigger");
                }
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {

            try{
                if(containingAtom.type == "UISlider")
                {
                    //Avoid overlap between min and max value
                    if(sliderMinJSON.val>=sliderMaxJSON.val)
                    {
                        sliderMinJSON.val=sliderMaxJSON.val;
                    }
                    if(pushDelay>0) //If delay is set
                    {
                        pushDelay-=0.01f;
                        sliderValue.GetFloatJSONParam("value").val+=sliderChangeJSON.val/100;
                    }//Push interpolation
                    else if((sliderValue.GetFloatJSONParam("value").val<pushFinalVal)&&(sliderValue.GetFloatJSONParam("value").val!=sliderMaxJSON.val))
                    {
                        sliderValue.GetFloatJSONParam("value").val=Mathf.Lerp(sliderValue.GetFloatJSONParam("value").val, sliderValue.GetFloatJSONParam("value").val+sliderPushJSON.val, sliderQuickJSON.val/10);
                    }
                    else//Constant change
                    {
                        sliderValue.GetFloatJSONParam("value").val+=sliderChangeJSON.val/100;
                        pushFinalVal=0;
                    }
                    
                    //stop change at min
                    var sliderValueVal=sliderValue.GetFloatJSONParam("value").val;
                    if(sliderValueVal<=sliderMinJSON.val)
                    {
                        sliderValue.GetFloatJSONParam("value").val=sliderMinJSON.val;
                    }

                    //stop change at max
                    if(sliderValueVal>=sliderMaxJSON.val)
                    {
                        sliderValue.GetFloatJSONParam("value").val=sliderMaxJSON.val;
                    }
                    currentValueJSON.val=sliderValue.GetFloatJSONParam("value").val;
                }
                
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

        protected void Push_Slider_Foo()
        {
            //sliderValue.GetFloatJSONParam("value").val+=sliderPushJSON.val;
            //timer=1-sliderQuickJSON.val;
            //pushInitialVal=sliderValue.GetFloatJSONParam("value").val;
            pushDelay=delayValueJSON.val;
            pushFinalVal=sliderValue.GetFloatJSONParam("value").val+sliderPushJSON.val;
        }

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

	}
}